from datetime import datetime
import bisect

class IntervalError(Exception):
    pass

class Interval(object):
    """Represents an interval of time"""

    def __init__(self, start, end):
        self.start = start
        self.end = end

    def __repr__(self):
        return "Interval(" + repr(self.start) + ", " + repr(self.end) + ")"

    def __str__(self):
        return "[" + str(self.start) + " -> " + str(self.end) + "]"

    def __getattr__(self, name):
        return { }

    def __setattr__(self, name, value):
        if (type(value) is not datetime):
            raise IntervalError("Must set datetime values")
        self.__dict__[name] = value
        if (type(self.start) is type(self.end)):
            if (self.start > self.end):
                raise IntervalError("Interval start must precede interval end")

    def __cmp__(self, other):
        """Compare two intervals.  If non-equal, order determined by start then end"""
        if (self.start == other.start):
            if (self.end < other.end):
                return -1
            if (self.end > other.end):
                return 1
            return 0
        if (self.start < other.start):
            return -1
        return 1
                
    def intersects(self, other):
        if (self.end <= other.start or
            self.start >= other.end):
            return False;
        else:
            return True;

 #                                start-other.start start-other.end  end-other.start  end-other.end
 #    ----                              <                <              >                <
 #      ----
 #
 # ----                                 <                <              <               <       n
 #      -----
 #
 #      ---                             >                 <             >               <
 #    --------
 #
 #    -------                           <                 <             >               >
 #      ---
 #
 #         ----                         >                 <             >               >
 #      -----
 #
 #              -----------             >                 >             >               >       n
 #      -----


class IntervalSet(object):
    """A non-intersecting set of intervals

    Kept sorted internally"""

    def __init__(self, iterable=None):
        self.data = []
        if type(iterable) is not None:
            if type(iterable) is Interval:
                iterable = [iterable]
            self.add_intervals(iterable)

    def add_intervals(self, iterable):
        for element in iter(iterable):
            self.add_single_interval(element)

    def add_single_interval(self, interval):
        for existing in self.data:
            if existing.intersects(interval):
                raise IntervalError("Tried to add overlapping interval to this set")
        bisect.insort(self.data, interval)
