from nilmdb import Interval, IntervalSet, IntervalError
from datetime import datetime
from nose.tools import assert_raises
import itertools 

def test_interval():
    """Test the Interval class"""
    d1 = datetime.strptime("19801205","%Y%m%d")
    d2 = datetime.strptime("19900216","%Y%m%d")
    d3 = datetime.strptime("20111205","%Y%m%d")

    # basic construction
    i = Interval(d1, d1)
    i = Interval(d1, d3)
    assert(i.start == d1)
    assert(i.end == d3)

    # assignment should work
    i.start = d2
    try:
        i.end = d1
        raise Exception("should have bombed out there")
    except IntervalError:
        pass

    # end before start
    assert_raises(IntervalError, Interval, d3, d1)

    # wrong type
    assert_raises(IntervalError, Interval, 1, 2)

    # compare
    assert(Interval(d1, d2) == Interval(d1, d2))
    assert(Interval(d1, d2) < Interval(d1, d3))
    assert(Interval(d1, d2) < Interval(d2, d3))
    assert(Interval(d1, d3) < Interval(d2, d3))
    assert(Interval(d2, d2) > Interval(d1, d3))
    assert(Interval(d3, d3) == Interval(d3, d3))

def test_interval_intersect():
    """Test Interval intersections"""
    dates = [ datetime.strptime(year, "%y") for year in [ "00", "01", "02", "03" ] ]
    perm = list(itertools.permutations(dates, 2))
    prod = list(itertools.product(perm, perm))
    should_intersect = {
        False: [4, 5, 8, 20, 48, 56, 60, 96, 97, 100],
        True: [0, 1, 2, 12, 13, 14, 16, 17, 24, 25, 26, 28, 29,
               32, 49, 50, 52, 53, 61, 62, 64, 65, 68, 98, 101, 104]}
    for i,((a,b),(c,d)) in enumerate(prod):
        try:
            i1 = Interval(a, b)
            i2 = Interval(c, d)
            assert(i1.intersects(i2) == i2.intersects(i1))
            assert(i in should_intersect[i1.intersects(i2)])
        except IntervalError:
            assert(i not in should_intersect[True] and
                   i not in should_intersect[False])

def test_intervalset():
    """Test interval sets"""

    d1 = datetime.strptime("19801205","%Y%m%d")
    d2 = datetime.strptime("19900216","%Y%m%d")
    d3 = datetime.strptime("20111205","%Y%m%d")

    a = Interval(d1, d2)
    b = Interval(d2, d3)
    c = Interval(d1, d3)
    
    iset = IntervalSet(a)
    iset = IntervalSet([a, b])
    assert_raises(IntervalError, IntervalSet, [a, b, c])
    
    iset = IntervalSet(iset)
